@@ -124,11 +124,12 @@ on_number_button_clicked               (GtkToggleButton  *button,
 {	
 	if (gtk_toggle_button_get_active(button) == FALSE) return;
 	button_activation (button);
-	if (current_status.notation == CS_FORMULA)
+	if (current_status.notation == CS_FORMULA) {
 		ui_formula_entry_insert (gtk_button_get_label ((GtkButton *)button));
-	else
+	} else {
 		rpn_stack_lift();
 		display_result_add_digit (*(gtk_button_get_label ((GtkButton *)button)));
+	}
 	return;
 }
 
@@ -167,7 +168,7 @@ on_operation_button_clicked            (GtkToggleButton       *button,
 	current_token.operation = (int) g_object_get_data (G_OBJECT (button), "operation");
 	/* current number, get it from the display! */
 	current_token.number = display_result_get_double ();
-	current_token.func = NULL;	
+	current_token.func = NULL;
 	/* do inverse left shift is a right shift */
 	if ((current_token.operation == '<') && \
 		(BIT (current_status.fmod, CS_FMOD_FLAG_INV) == 1)) {
@@ -188,7 +189,6 @@ on_operation_button_clicked            (GtkToggleButton       *button,
 		 * in general, a closing bracket is only useful if there were opening
 		 *	brackets.
 		 */
-		
 		if (((current_token.operation == '(') || current_status.allow_arith_op) && \
 			((current_token.operation != ')') || (display_module_bracket_label_update (GET) > 0))) {
 			return_value = alg_add_token (&main_alg, current_token);
@@ -576,8 +576,17 @@ void
 on_paste_activate (GtkMenuItem     *menuitem,
 			gpointer         user_data)
 {
-	display_result_feed (gtk_clipboard_wait_for_text (
-		gtk_clipboard_get (GDK_SELECTION_CLIPBOARD)));
+	GtkWidget	*formula_entry;
+	char		*cp_text;
+	
+	cp_text = gtk_clipboard_wait_for_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD));
+	if (cp_text) {
+		if ((formula_entry = formula_entry_is_active_no_toplevel_check ()) != NULL) {
+			gtk_editable_paste_clipboard((GtkEditable *)formula_entry);
+		}
+		else display_result_feed (cp_text);
+		g_free (cp_text);
+	}
 }
 
 void
@@ -933,25 +942,7 @@ void on_prefs_rem_display_toggled (GtkToggleButton *togglebutton,
 	prefs.rem_display = gtk_toggle_button_get_active (togglebutton);
 	/* only is important when leaving galculator */
 }
-/*
-void on_prefs_def_number_changed (GtkOptionMenu *optionmenu,
-				gpointer user_data)
-{
-	prefs.def_number = gtk_option_menu_get_history (optionmenu);
-}
-										
-void on_prefs_def_angle_changed (GtkOptionMenu *optionmenu,
-				gpointer user_data)
-{
-	prefs.def_angle = gtk_option_menu_get_history (optionmenu);
-}
-										
-void on_prefs_def_notation_changed (GtkOptionMenu *optionmenu,
-				gpointer user_data)
-{
-	prefs.def_notation = gtk_option_menu_get_history (optionmenu);
-}
-*/
+
 void on_prefs_button_width_changed (GtkSpinButton *spinbutton,
 					GtkScrollType arg1,
 					gpointer user_data)
@@ -1447,6 +1438,138 @@ void on_prefs_bin_length_value_changed (GtkSpinButton *spinbutton,
 	prefs.bin_length = (int) gtk_spin_button_get_value (spinbutton);
 }
 
+void on_prefs_menu_dec_activate (GtkMenuItem *menuitem, gpointer user_data)
+{
+	gtk_widget_show (glade_xml_get_widget (prefs_xml, "prefs_vbox_dec"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_hex"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_oct"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_bin"));
+}
+			
+void on_prefs_menu_hex_activate (GtkMenuItem *menuitem, gpointer user_data)
+{
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_dec"));
+	gtk_widget_show (glade_xml_get_widget (prefs_xml, "prefs_vbox_hex"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_oct"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_bin"));
+}
+			
+void on_prefs_menu_oct_activate (GtkMenuItem *menuitem, gpointer user_data)
+{
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_dec"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_hex"));
+	gtk_widget_show (glade_xml_get_widget (prefs_xml, "prefs_vbox_oct"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_bin"));
+}
+			
+void on_prefs_menu_bin_activate (GtkMenuItem *menuitem, gpointer user_data)
+{
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_dec"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_hex"));
+	gtk_widget_hide (glade_xml_get_widget (prefs_xml, "prefs_vbox_oct"));
+	gtk_widget_show (glade_xml_get_widget (prefs_xml, "prefs_vbox_bin"));
+}
+
+void on_prefs_dec_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data)
+{
+	prefs.dec_sep = gtk_toggle_button_get_active (togglebutton);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_dec_sep_char_label"), prefs.dec_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_dec_sep_char"), prefs.dec_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_dec_sep_length_label"), prefs.dec_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_dec_sep_length"), prefs.dec_sep);
+	display_result_getset();
+}
+
+void on_prefs_hex_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data)
+{
+	prefs.hex_sep = gtk_toggle_button_get_active (togglebutton);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_hex_sep_char_label"), prefs.hex_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_hex_sep_char"), prefs.hex_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_hex_sep_length_label"), prefs.hex_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_hex_sep_length"), prefs.hex_sep);
+	display_result_getset();
+}
+
+void on_prefs_oct_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data)
+{
+	prefs.oct_sep = gtk_toggle_button_get_active (togglebutton);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_oct_sep_char_label"), prefs.oct_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_oct_sep_char"), prefs.oct_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_oct_sep_length_label"), prefs.oct_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_oct_sep_length"), prefs.oct_sep);
+	display_result_getset();
+}
+
+void on_prefs_bin_sep_toggled (GtkToggleButton *togglebutton, 
+					gpointer user_data)
+{
+	prefs.bin_sep = gtk_toggle_button_get_active (togglebutton);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_bin_sep_char_label"), prefs.bin_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_bin_sep_char"), prefs.bin_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_bin_sep_length_label"), prefs.bin_sep);
+	gtk_widget_set_sensitive (glade_xml_get_widget (prefs_xml, "prefs_bin_sep_length"), prefs.bin_sep);
+	display_result_getset();
+}
+
+void on_prefs_dec_sep_length_value_changed (GtkSpinButton *spinbutton,
+					GtkScrollType arg1,
+					gpointer user_data)
+{
+	prefs.dec_sep_length = (int) gtk_spin_button_get_value (spinbutton);
+	display_result_getset();
+}
+
+void on_prefs_hex_sep_length_value_changed (GtkSpinButton *spinbutton,
+					GtkScrollType arg1,
+					gpointer user_data)
+{
+	prefs.hex_sep_length = (int) gtk_spin_button_get_value (spinbutton);
+	display_result_getset();
+}
+
+void on_prefs_oct_sep_length_value_changed (GtkSpinButton *spinbutton,
+					GtkScrollType arg1,
+					gpointer user_data)
+{
+	prefs.oct_sep_length = (int) gtk_spin_button_get_value (spinbutton);
+	display_result_getset();
+}
+
+void on_prefs_bin_sep_length_value_changed (GtkSpinButton *spinbutton,
+					GtkScrollType arg1,
+					gpointer user_data)
+{
+	prefs.bin_sep_length = (int) gtk_spin_button_get_value (spinbutton);
+	display_result_getset();
+}
+
+void on_prefs_dec_sep_char_changed (GtkEditable *editable,
+                                            gpointer user_data)
+{
+	prefs_sep_char_changed (editable, prefs.dec_sep_char, CS_DEC);
+}
+
+void on_prefs_hex_sep_char_changed (GtkEditable *editable,
+                                            gpointer user_data)
+{
+	prefs_sep_char_changed (editable, prefs.hex_sep_char, CS_HEX);
+}
+
+void on_prefs_oct_sep_char_changed (GtkEditable *editable,
+                                            gpointer user_data)
+{
+	prefs_sep_char_changed (editable, prefs.oct_sep_char, CS_OCT);
+}
+
+void on_prefs_bin_sep_char_changed (GtkEditable *editable,
+                                            gpointer user_data)
+{
+	prefs_sep_char_changed (editable, prefs.bin_sep_char, CS_BIN);
+}
+
 void on_togglebutton_released (GtkToggleButton *togglebutton, 
 					gpointer user_data)
 {
